@@ -1,4 +1,7 @@
+from typing import Any
+
 import os
+import logging
 import textwrap
 from abc import ABC
 from copy import deepcopy
@@ -13,11 +16,16 @@ from mergedeep import merge
 from yamlpath.common import Parsers
 from yamlpath.wrappers import ConsolePrinter
 
-from gitlabform import EXIT_INVALID_INPUT
+from gitlabform.constants import EXIT_INVALID_INPUT
 from ruamel.yaml.comments import CommentedMap
 
 
 class ConfigurationCore(ABC):
+    """
+    The functionality shared by various types of config (common, groups, projects)
+    is implemented here.
+    """
+
     def __init__(self, config_path=None, config_string=None):
 
         if config_path and config_string:
@@ -54,10 +62,12 @@ class ConfigurationCore(ABC):
                         exit_code=EXIT_INVALID_INPUT,
                     )
 
-                # we are NOT checking for the existence of non-empty 'projects_and_groups' key here
-                # as it would break using GitLabForm as a library
+            self._find_almost_duplicates()
 
-        except (FileNotFoundError, IOError):
+            # we are NOT checking for the existence of non-empty 'projects_and_groups' key here
+            # as it would break using GitLabForm as a library
+
+        except (FileNotFoundError, OSError):
             raise ConfigFileNotFoundException(config_path)
 
         except Exception as e:
@@ -92,10 +102,10 @@ class ConfigurationCore(ABC):
         yaml.version = (1, 1)
 
         if config_string:
-            config_string = textwrap.dedent(source)
+            config = textwrap.dedent(source)
             verbose("Reading config from the provided string.")
             (yaml_data, doc_loaded) = Parsers.get_yaml_data(
-                yaml, log, config_string, literal=True
+                yaml, log, config, literal=True
             )
         else:
             config_path = source
@@ -110,7 +120,7 @@ class ConfigurationCore(ABC):
 
         return yaml_data
 
-    def get(self, path, default=None):
+    def get(self, path, default=None) -> Any:
         """
         :param path: "path" to given element in YAML file, for example for:
 
@@ -136,44 +146,23 @@ class ConfigurationCore(ABC):
         try:
             for token in tokens:
                 current = current[token]
+
+            if isinstance(current, ScalarString):
+                to_return = str(current)
+            else:
+                to_return = current
         except:
             if default is not None:
-                return default
+                to_return = default
             else:
-                raise KeyNotFoundException
+                raise KeyNotFoundException(path) from None
 
-        if isinstance(current, ScalarString):
-            return str(current)
-        else:
-            return current
-
-    def get_group_config(self, group) -> dict:
-        pass
-
-    def get_project_config(self, group_and_project) -> dict:
-        pass
-
-    def get_common_config(self) -> dict:
-        """
-        :return: literal common configuration or empty dict if not defined
-        """
-        common_config = self.get("projects_and_groups|*", {})
-        if common_config:
-            section_name = "*"
-            self.validate_break_inheritance_flag(common_config, section_name)
-        return common_config
-
-    def is_project_skipped(self, project) -> bool:
-        pass
-
-    def is_group_skipped(self, group) -> bool:
-        pass
-
-    def is_skipped(self, an_array: list, item: str) -> bool:
-        pass
+        return to_return
 
     @staticmethod
-    def validate_break_inheritance_flag(config, section_name, parent_key=""):
+    def _validate_break_inheritance_flag(
+        config: dict, section_name: str, parent_key: str = ""
+    ) -> None:
         for key, value in config.items():
             if "inherit" == key:
                 parent_key_description = (
@@ -184,13 +173,13 @@ class ConfigurationCore(ABC):
                     f"because it has no higher level setting to inherit from.\n",
                     exit_code=EXIT_INVALID_INPUT,
                 )
-            elif type(value) is CommentedMap:
-                ConfigurationCore.validate_break_inheritance_flag(
+            elif type(value) in [CommentedMap, dict]:
+                ConfigurationCore._validate_break_inheritance_flag(
                     value, section_name, key
                 )
 
     @staticmethod
-    def merge_configs(more_general_config, more_specific_config) -> dict:
+    def _merge_configs(more_general_config, more_specific_config) -> dict:
         """
         :return: merge more general config with more specific configs.
                  More specific config values take precedence over more general ones.
@@ -214,7 +203,7 @@ class ConfigurationCore(ABC):
                             f"Cannot set the inheritance break flag with true\n",
                             exit_code=EXIT_INVALID_INPUT,
                         )
-                elif type(value) is CommentedMap:
+                elif type(value) in [CommentedMap, dict]:
                     break_inheritance(value, key)
 
         def replace_config_sections(merged_config, specific_key, specific_config):
@@ -223,13 +212,102 @@ class ConfigurationCore(ABC):
                     del specific_config["inherit"]
                     merged_config[key] = specific_config
                     break
-                elif type(value) is CommentedMap:
+                elif type(value) in [CommentedMap, dict]:
                     replace_config_sections(value, specific_key, specific_config)
 
         break_inheritance(more_specific_config)
 
         return dict(merged_dict)
 
+    @staticmethod
+    def _get_case_insensitively(a_dict: dict, a_key: str):
+        for dict_key in a_dict.keys():
+            if dict_key.lower() == a_key.lower():
+                return a_dict[dict_key]
+        return {}
+
+    @staticmethod
+    def _is_skipped_case_insensitively(an_array: list, item: str) -> bool:
+        """
+        :return: if item is defined in the list to be skipped
+        """
+        item = item.lower()
+
+        for list_element in an_array:
+            list_element = list_element.lower()
+
+            if list_element == item:
+                return True
+
+            if (
+                list_element.endswith("/*")
+                and item.startswith(list_element[:-2])
+                and len(item) >= len(list_element[:-2])
+            ):
+                return True
+
+        return False
+
+    def _find_almost_duplicates(self):
+
+        # in GitLab groups and projects names are de facto case insensitive:
+        # you can change the case of both name and path BUT you cannot create
+        # 2 groups which names differ only with case and the same thing for
+        # projects. therefore we cannot allow such entries in the config,
+        # as they would be ambiguous.
+
+        for path in [
+            "projects_and_groups",
+            "skip_groups",
+            "skip_projects",
+        ]:
+            if self.get(path, 0):
+                almost_duplicates = self._find_almost_duplicates_in(path)
+                if almost_duplicates:
+                    fatal(
+                        f"There are almost duplicates in the keys of {path} - they differ only in case.\n"
+                        f"They are: {', '.join(almost_duplicates)}\n"
+                        f"This is not allowed as we ignore the case for group and project names.",
+                        exit_code=EXIT_INVALID_INPUT,
+                    )
+
+    def _find_almost_duplicates_in(self, configuration_path):
+        """
+        Checks given configuration key and reads its keys - if it is a dict - or elements - if it is a list.
+        Looks for items that are almost the same - they differ only in the case.
+        :param configuration_path: configuration path, f.e. "group_settings"
+        :return: list of items that have almost duplicates,
+                 or an empty list if none are found
+        """
+
+        dict_or_list = self.get(configuration_path)
+        if isinstance(dict_or_list, dict):
+            items = dict_or_list.keys()
+        else:
+            items = dict_or_list
+
+        items_with_lowercase_names = [x.lower() for x in items]
+
+        # casting these to sets will deduplicate the one with lowercase names
+        # lowering its cardinality if there were elements in it
+        # that before lowering differed only in case
+        if len(set(items)) != len(set(items_with_lowercase_names)):
+
+            # we have some almost duplicates, let's find them
+            almost_duplicates = []
+            for first_item in items:
+                occurrences = 0
+                for second_item in items_with_lowercase_names:
+                    if first_item.lower() == second_item.lower():
+                        occurrences += 1
+                        if occurrences == 2:
+                            almost_duplicates.append(first_item)
+                            break
+            return almost_duplicates
+
+        else:
+            return []
+
 
 class ConfigFileNotFoundException(Exception):
     pass
@@ -241,4 +319,13 @@ class ConfigInvalidException(Exception):
 
 
 class KeyNotFoundException(Exception):
-    pass
+    __slots__ = "key"
+
+    def __init__(self, key: str):
+        if logging.getLogger().getEffectiveLevel() <= logging.DEBUG:
+            self.key = key
+        else:
+            fatal(
+                f"Unable to find the key: {key.replace('|', '.')}\n",
+                exit_code=EXIT_INVALID_INPUT,
+            )
