@@ -2,27 +2,7 @@ from gitlabform.gitlab.core import GitLabCore, NotFoundException
 
 
 class GitLabBranches(GitLabCore):
-
-    # old API
-    def protect_branch(
-        self, project_and_group_name, branch, developers_can_push, developers_can_merge
-    ):
-        data = {
-            "id": project_and_group_name,
-            "branch": branch,
-            "developers_can_push": developers_can_push,
-            "developers_can_merge": developers_can_merge,
-        }
-        return self._make_requests_to_api(
-            "projects/%s/repository/branches/%s/protect",
-            (project_and_group_name, branch),
-            method="PUT",
-            data=data,
-            expected_codes=[200, 201],
-        )
-
-    # new API
-    def branch_access_level(self, project_and_group_name, branch, protect_settings):
+    def protect_branch(self, project_and_group_name, branch, protect_settings):
 
         url = "projects/%s/protected_branches?name=%s"
         parameters_list = [
@@ -33,16 +13,14 @@ class GitLabBranches(GitLabCore):
             url,
             tuple(parameters_list),
             method="POST",
-            # TODO: check why is 409 Conflict accepted here :/
             expected_codes=[
                 200,
                 201,
-                409,
             ],
             json=protect_settings,
         )
 
-    def branch_code_owner_approval_required(
+    def set_branch_code_owner_approval_required(
         self,
         project_and_group_name,
         branch,
@@ -65,19 +43,6 @@ class GitLabBranches(GitLabCore):
         )
 
     def unprotect_branch(self, project_and_group_name, branch):
-        data = {
-            "id": project_and_group_name,
-            "branch": branch,
-        }
-        return self._make_requests_to_api(
-            "projects/%s/repository/branches/%s/unprotect",
-            (project_and_group_name, branch),
-            method="PUT",
-            data=data,
-            expected_codes=[200, 201],
-        )
-
-    def unprotect_branch_new_api(self, project_and_group_name, branch):
         # 404 means that the branch is already unprotected
         return self._make_requests_to_api(
             "projects/%s/protected_branches/%s",
@@ -88,7 +53,7 @@ class GitLabBranches(GitLabCore):
 
     def get_branches(self, project_and_group_name):
         result = self._make_requests_to_api(
-            "projects/%s/repository/branches", project_and_group_name, paginated=True
+            "projects/%s/repository/branches", project_and_group_name
         )
         return sorted(map(lambda x: x["name"], result))
 
@@ -102,9 +67,6 @@ class GitLabBranches(GitLabCore):
             "projects/%s/protected_branches/%s", (project_and_group_name, branch)
         )
 
-    def get_user_to_protect_branch(self, user_name):
-        return self._get_user_id(user_name)
-
     def get_only_branch_access_levels(self, project_and_group_name, branch):
         try:
             result = self._make_requests_to_api(
@@ -174,21 +136,14 @@ class GitLabBranches(GitLabCore):
         )
 
     def get_protected_branches(self, project_and_group_name):
-        branches = self._make_requests_to_api(
-            "projects/%s/repository/branches", project_and_group_name, paginated=True
+        protected_branches = self._make_requests_to_api(
+            "projects/%s/protected_branches", project_and_group_name
         )
-
-        protected_branches = []
-        for branch in branches:
-            if branch["protected"]:
-                name = branch["name"]
-                protected_branches.append(name)
-
-        return protected_branches
+        return [branch["name"] for branch in protected_branches]
 
     def get_unprotected_branches(self, project_and_group_name):
         branches = self._make_requests_to_api(
-            "projects/%s/repository/branches", project_and_group_name, paginated=True
+            "projects/%s/repository/branches", project_and_group_name
         )
 
         unprotected_branches = []
