@@ -3,9 +3,9 @@ from cli_ui import debug as verbose
 from cli_ui import fatal
 
 import abc
-from typing import Callable, List
+from typing import Callable, Union, Any
 
-from gitlabform import EXIT_INVALID_INPUT
+from gitlabform.constants import EXIT_INVALID_INPUT
 from gitlabform.gitlab import GitLab
 from gitlabform.processors.abstract_processor import AbstractProcessor
 from gitlabform.processors.defining_keys import AbstractKey
@@ -16,33 +16,54 @@ class MultipleEntitiesProcessor(AbstractProcessor, metaclass=abc.ABCMeta):
         self,
         configuration_name: str,
         gitlab: GitLab,
-        list_method_name: str,
-        add_method_name: str,
-        delete_method_name: str,
+        list_method_name: Union[str, Callable[[str], list]],
+        add_method_name: Union[str, Callable[[str, Any], None]],
+        delete_method_name: Union[str, Callable[[str, dict], None]],
         defining: AbstractKey,
         required_to_create_or_update: AbstractKey,
-        edit_method_name=None,
+        edit_method_name: Union[str, Callable[[str, dict, dict], None], None] = None,
     ):
+        """
+        :param configuration_name: The key in the YAML cfg file that determines the start of this Entity's section
+        :param gitlab: Gitlab instance
+        :param list_method_name: Retrieves the entities in question
+        :param add_method_name: Creates a new entity in Gitlab
+        :param delete_method_name: Removes the entity from Gitlab
+        :param defining: An expression that tells which fields of the entity in GitLab are identifying it.
+        :param required_to_create_or_update: Entries which (according to the API documentation) are required fields
+        :param edit_method_name: Edits the existing entity in Gitlab
+        """
+
         super().__init__(configuration_name, gitlab)
-        self.list_method: Callable = getattr(self.gitlab, list_method_name)
-        self.add_method: Callable = getattr(self.gitlab, add_method_name)
-        self.delete_method: Callable = getattr(self.gitlab, delete_method_name)
+        self.list_method: Callable = (
+            getattr(self.gitlab, list_method_name)
+            if (isinstance(list_method_name, str))
+            else list_method_name
+        )
+        self.add_method: Callable = (
+            getattr(self.gitlab, add_method_name)
+            if (isinstance(add_method_name, str))
+            else add_method_name
+        )
+        self.delete_method: Callable = (
+            getattr(self.gitlab, delete_method_name)
+            if (isinstance(delete_method_name, str))
+            else delete_method_name
+        )
         self.defining: AbstractKey = defining
         self.required_to_create_or_update: AbstractKey = required_to_create_or_update
 
         if edit_method_name:
-            self.edit_method: Callable = getattr(self.gitlab, edit_method_name)
+            self.edit_method: Union[Callable, None] = (
+                getattr(self.gitlab, edit_method_name)
+                if (isinstance(edit_method_name, str))
+                else edit_method_name
+            )
         else:
             self.edit_method = None
 
-    def _can_proceed(self, project_or_group: str, configuration: dict):
-        return True
-
     def _process_configuration(self, project_or_group: str, configuration: dict):
 
-        if not self._can_proceed(project_or_group, configuration):
-            return
-
         entities_in_configuration = configuration[self.configuration_name]
         if "enforce" in entities_in_configuration:
             enforce = entities_in_configuration["enforce"]
@@ -91,7 +112,7 @@ class MultipleEntitiesProcessor(AbstractProcessor, metaclass=abc.ABCMeta):
                 # no need to validate if we have what's needed to delete as we got the entities from gitlab
                 verbose(
                     f"Deleting entity no {entity_name} of {self.configuration_name} in {project_or_group} "
-                    f" as it's not in config and enforce is set to true."
+                    f"as it's not in config and enforce is set to true."
                 )
                 self.delete_method(project_or_group, entity_config)
 
@@ -120,14 +141,14 @@ class MultipleEntitiesProcessor(AbstractProcessor, metaclass=abc.ABCMeta):
                     debug(f"{self.configuration_name} AFTER: ^^^")
                 else:
                     verbose(
-                        f"Recreating {entity_name} of {self.configuration_name} in {project_or_group}"
+                        f" * Recreating {entity_name} of {self.configuration_name} in {project_or_group}"
                     )
                     self.delete_method(project_or_group, entity_in_gitlab)
                     self.add_method(project_or_group, entity_config)
                     debug(f"{self.configuration_name} AFTER: ^^^")
             else:
                 verbose(
-                    f"{entity_name} of {self.configuration_name} in {project_or_group} doesn't need an update."
+                    f" * {entity_name} of {self.configuration_name} in {project_or_group} doesn't need an update."
                 )
 
         # add c) (or do nothing if marked as "delete")
@@ -138,7 +159,7 @@ class MultipleEntitiesProcessor(AbstractProcessor, metaclass=abc.ABCMeta):
                 project_or_group, entity_name, entity_config
             )
             verbose(
-                f"Adding {entity_name} of {self.configuration_name} in {project_or_group}"
+                f" * Adding {entity_name} of {self.configuration_name} in {project_or_group}"
             )
             self.add_method(project_or_group, entity_config)
             debug(f"{self.configuration_name} AFTER: ^^^")
