@@ -2,7 +2,7 @@ from logging import debug
 from cli_ui import debug as verbose
 from cli_ui import fatal
 
-from gitlabform import EXIT_INVALID_INPUT
+from gitlabform.constants import EXIT_INVALID_INPUT
 from gitlabform.gitlab import GitLab
 from gitlabform.processors.abstract_processor import AbstractProcessor
 
@@ -109,23 +109,31 @@ class MembersProcessor(AbstractProcessor):
                     else None
                 )
                 # we only add the user if it doesn't have the correct settings
-                if (
-                    user in current_members
-                    and expires_at == current_members[user]["expires_at"]
-                    and access_level == current_members[user]["access_level"]
-                ):
-                    debug("Ignoring user '%s' as it is already a member", user)
-                    debug(
-                        "Current settings for '%s' are: %s"
-                        % (user, current_members[user])
-                    )
+                if user in current_members:
+                    if (
+                        expires_at == current_members[user]["expires_at"]
+                        and access_level == current_members[user]["access_level"]
+                    ):
+                        debug(
+                            "Nothing to change for user '%s' - same config now as to set.",
+                            user,
+                        )
+                        debug(
+                            "Current settings for '%s' are: %s"
+                            % (user, current_members[user])
+                        )
+                    else:
+                        debug(
+                            "Editing user '%s' membership to change their access level or expires at",
+                            user,
+                        )
+                        self.gitlab.edit_member_of_project(
+                            project_and_group, user, access_level, expires_at
+                        )
                 else:
-                    debug("Setting user '%s' as a member", user)
-                    access = access_level
-                    expiry = expires_at
-                    self.gitlab.remove_member_from_project(project_and_group, user)
+                    debug("Adding user '%s' who previously was not a member.", user)
                     self.gitlab.add_member_to_project(
-                        project_and_group, user, access, expiry
+                        project_and_group, user, access_level, expires_at
                     )
 
         if enforce_members:
