@@ -16,6 +16,7 @@ from tests.acceptance import (
     get_random_name,
     create_users,
     delete_users,
+    get_random_password,
 )
 
 
@@ -82,7 +83,7 @@ def third_group():
 
 
 @pytest.fixture(scope="class")
-def sub_group(group):
+def subgroup(group):
     gl = get_gitlab()
     parent_id = gl.get_group_id_case_insensitive(group)
     group_name = get_random_name("subgroup")
@@ -94,6 +95,17 @@ def sub_group(group):
     gl.delete_group(group + "/" + group_name)
 
 
+@pytest.fixture(scope="class")
+def project_in_subgroup(subgroup):
+    project_name = get_random_name("project")
+    create_project(subgroup, project_name)
+
+    yield f"{subgroup}/{project_name}"
+
+    gl = get_gitlab()
+    gl.delete_project(f"{subgroup}/{project_name}")
+
+
 @pytest.fixture(scope="class")
 def project(group):
     project_name = get_random_name("project")
@@ -205,13 +217,16 @@ def make_user(
         last_id = len(created_users) + 1
         username = f"{username_base}_{last_id}"
         user = gitlab.create_user(
-            username + "@example.com", username + " Example", username, "password"
+            username + "@example.com",
+            username + " Example",
+            username,
+            get_random_password(),
         )
-        user_obj = User(username, user["id"])
+
         if add_to_project:
-            gitlab.add_member_to_project(
-                group_and_project, None, level.value, user_id=user["id"]
-            )
+            gitlab.add_member_to_project(group_and_project, username, level.value)
+
+        user_obj = User(username, user["id"])
         created_users.append(user_obj)
         return user_obj
 
@@ -234,7 +249,10 @@ def three_members(gitlab, group_and_project, make_user):
 def outsider_user(gitlab, group_and_project):
     username = get_random_name("outsider_user")
     user = gitlab.create_user(
-        username + "@example.com", username + " Example", username, "password"
+        username + "@example.com",
+        username + " Example",
+        username,
+        get_random_password(),
     )
     user_obj = User(username, user["id"])
 
